#ifndef __filesystem_include__
#define __filesystem_include__

#include <sys/types.h>

#include "FSResult.hh"
#include "FSDirectory.hh"
#include "FSFile.hh"
#include "FSLogic.hh"

#define MAXFD 10

class FileSystem {
protected:
    /** number of last error */
    unsigned int _fserrno;
    /** device that implements the file system logic */
    FSLogic* _logic;
    /** current working directory */
    FSDirectory* _current;
    /** array of file objects associate with their descriptor ids */
    FSFile* _descriptors[MAXFD];
    
    /**
     * Computes for a specified path the absolut path.
     *
     * @param path path to a filesystem object
     * @return Absolut path to the filesystem object.
     */
    char* absolutPath(const char *path);
public:
    FileSystem(FSLogic *logic);
    ~FileSystem();

    /**
     * Prints the name of current working directory.
     */
    void pwd();
    
    /**
     * Changes the current directory to that specified in path. 
     *
     * @return On success, zero is returned.  On error, -1 is returned, and errno 
     *         is set appropriately.
     */
    int chdir(const char *path);
    
    /**
     * Attempts to create a directory named pathname.
     *
     * @return Returns zero on success, or -1 if an error occurred (in which case, 
     *         errno is set appropriately).
     */
    int mkdir(const char *path);
    
    /**
     * Deletes an empty directory specified by path.
     *
     * @return On success, zero is returned. On error, -1 is returned, and 
     *         errno is set appropriately.
     */
    int rmdir(const char *path);
    
    /**
     * Prints the directories/files of the current working directory.
     */
    int readdir();
    
    /**
     * Used to convert a pathname into a file descriptor for use in subsequent 
     * I/O as with read(2), write(2), etc.. Returns lowest file descriptor 
     * available if successful.     
     *
     * @param pathname path to the file to be opened
     * @param flags parameter flags O_RDONLY, O_WRONLY or O_RDWR
     * @return Returns the new file descriptor, or -1 if an error occurred 
     *         (in which case, errno is set appropriately).
     */
    int open(const char *pathname, int flags); 

    /**
     * Closes a file descriptor, so that it no longer refers to any file and may 
     * be reused.
     *
     * @param fd file descriptor to be closed
     * @return Returns zero on success. On error, -1 is returned, and errno is 
     *         set appropriately.
     */
    int close(int fd);

    /**
     * The lseek function repositions the file position of the open file 
     * associated with the file descriptor fd to the argument offset according 
     * to the directive whence as follows:
     *
     * SEEK_SET     The offset is set to offset bytes.
     * SEEK_CUR     The offset is set to its current location plus offset bytes.
     * SEEK_END     The offset is set to the size of the file plus offset bytes.
     *
     * @param fd file descriptor of the file
     * @param offset offset for reposition
     * @param whence counting position for offset (default is SEEK_CUR)
     * @return Upon successful completion, lseek returns the resulting offset location 
     *         as measured in bytes from the beginning of the file. Otherwise, a value
     *         of -1 is returned and errno is set to indicate the error.
     */
    int lseek(int fd, size_t offset, int whence);
   
    /**
     * Attempts to read up to count bytes from file descriptor fd into the buffer 
     * starting at buffer.
     *
     * @param fd file descriptor of the file
     * @param buffer pointer to readbuffer
     * @param count number of bytes to be read
     * @return On success, the  number of bytes read is returned (zero indicates end 
     *         of file), and the file position is advanced by this number. 
     *         It is not an error if this number is smaller than the number of bytes 
     *         requested. This may happen for example because fewer bytes are actually 
     *         available. On error, -1 is returned, and errno is set appropriately. 
     */
    int read(int fd, caddr_t buffer, size_t count);
    
    /**
     * Writes up to count bytes to the file referenced by the file descriptor fd from 
     * buffer starting at buffer.
     *
     * @param fd file descriptor of the file
     * @param buffer pointer to readbuffer
     * @param count number of bytes to be read
     * @return On success, the number of bytes written are returned (zero indicates 
     *         nothing was written). On error, -1 is returned, and errno is set 
     *         appropriately. If count is zero and the file descriptor refers to a 
     *         regular file, 0 will be returned without causing any other effect.  
     */
    int write(int fd, const caddr_t buffer, size_t count);
};

#endif // __filesystem_include__
