#ifndef _MemoryManager_interface_
#define _MemoryManager_interface_

#include <sys/types.h>
#include <string.h>

/**
 * Der <code>MemoryManager</code> verwaltet einen durch die Startadresse 
 * und Gre definierten Speicherbereich anhand einer definierten Speicherstrategie.
 */
class MemoryManager {
private:   
    /**
     * Definiert die Struktur, die den Namen einer Strategie auf eine konkrete Instanz abbildet.
     */
    struct strategymap {
        const char *name;
        class MemoryStrategy *strategy;
    };
    
    /**
     * Liste der mglichen Speicherstrategien.
     */
    static const strategymap strategytable[];
    
    /**
     * verwendete Strategie zur Verwaltung des Speicherbereiches
     */
    MemoryStrategy* _strategy;
public:
    /**
     * Erzeugt eine Instanz der Klasse MemoryManager.
     * 
     * @param astrategy Strategie mit der der Speicherbereich verwaltet werden soll.
     * @param memadr Startadresse des verwalteten Speicherbereichs
     * @param memsize Gre des verwalteten Speicherbereichs
     */
    MemoryManager(char* astrategy, caddr_t memadr, size_t memsize);
    ~MemoryManager();
    
    /**
     * The malloc() function allocates size bytes of memory. The allocated
     * space is suitably aligned (after possible pointer coercion) for storage
     * of any type of object. If the space is at least pagesize bytes in length
     * (see getpagesize()), the returned memory will be page boundary aligned
     * as well. If malloc() fails, a NULL pointer is returned.
     *
     * <strong>Hinweis:</strong> Im Normalfall initialisiert malloc() den 
     *                           allokierten Speicherbereich NICHT mit 0 Bytes.
     *
     * @param size Gre des zu allokierenden Speicherbereichs
     * @return Liefert die Adresse des allokierten Speicherbereichs oder NULL 
     *         wenn malloc() fehlschlgt.
     */
    void* malloc(size_t size);
    
    /**
     * The calloc() function allocates space for number objects, each size bytes
     * in length. The result is identical to calling malloc() with an argument
     * of <em>"number * size"</em>, with the exception that the allocated memory is
     * explicitly initialized to zero bytes.
     *
     * @param number Anzahl der zu allokierenden Speicherbereiche
     * @param size Gre eines zu allokierenden Speicherbereichs
     * @return Liefert die Adresse des allokierten Speicherbereichs oder NULL 
     *         wenn calloc() fehlschlgt.
     */
    void* calloc(size_t number, size_t size);
    
    /**
     * The realloc() function changes the size of previously allocated memory 
     * referenced by ptr to size bytes. The contents of the memory are
     * unchanged up to the lesser of the new and old sizes.  If the new size is
     * larger, the value of the newly allocated portion of the memory is 
     * undefined. If the requested memory cannot be allocated, NULL is returned
     * and the memory referenced by ptr is valid and unchanged. If memory can
     * be allocated, the memory referenced by ptr is freed and a pointer to the
     * newly allocated memory is returned. <strong>Note</strong> that this may
     * be different from the value passed as ptr. If ptr is NULL, the realloc()
     * function behaves identically to malloc() for the specified size.
     *
     * @param ptr Adresse des Speicherbereichs
     * @param size neue Gre des Speicherbereichs in Bytes
     * @return Liefert die Adresse des reallokierten Speicherbereichs oder NULL 
     *         wenn realloc() fehlschlgt.
     */
    void* realloc(void *ptr, size_t size);
    
    /**
     * Gibt eine allokierten Speicherbereich wieder frei. 
     * Die Adresse des freizugebenden Speicherbereichs wird als Parameter 
     * bergeben. Ist die Adresse gleich NULL wird keine Aktion ausgefhrt.
     *
     * @param ptr Adresse des Speicherbereichs
     */
    void free(void *ptr);    
    
    /**
     * Gibt die aktuelle Freispeicherlist auf dem 'ErrorStream' aus. 
     * Diese Methode dient nur zum Debuggen und Testen der Speicherstrategieen.
     */
    void showBlocks();

    /**
     * Gibt die Gre des zur Verfgung stehenden Speichers zurck.
     *
     * @return Liefert die Gre des verwalteten Speichers.
     */
    size_t size();    
    
    /**
     * Gibt die Gre des belegten Speichers zurck.
     *
     * @return Liefert die Gre des belegten Speichers
     */
    size_t used();    
};

#endif // _MemoryManager_interface_
