#ifndef _MemoryStrategy_interface_
#define _MemoryStrategy_interface_

#include <sys/types.h>

/**
 * The base of all strategies that can be used by a MemoryManager.
 *
 * depending on the configuration this class is either empty
 * or it provides virtual functions get() and release()
 * In any case it is the base of an allocation strategy
 */
class MemoryStrategy {
private:
protected:    
    /**
     * Startadresse des verwalteten Speichers.
     */
    caddr_t _memadr;
    
    /**
     * Gre des verwalteten Speichers.
     */
    size_t _memsize;
    
    /**
     * wird in Bit angegeben um nur schiften zu verwenden
     */
    unsigned char _alignment;
    
    /**
     * Gibt fr eine Speichergre die nchste alignte Gre zurck.
     */
    size_t align(size_t wert);
    
    /**
     * Berechnet das Alignment anhand der Alignmentgre.
     */
    size_t getAlignment(size_t alignment);
    
    /**
     * Gibt an wieviel Speicher bereits verwendet wird.
     */
    size_t _used;               // wird modifiziert, wenn speicher angefordert oder freigegeben wird
    
    /**
     * Belegt einen Speicherbereich der angeforderten Gre.
     *
     * @param size Gre des angeforderten Speicherbereichs
     * @return Liefert die Adresse des angeforderten Speicherbereichs.
     */
    virtual void* get(size_t size) = 0;
    
    /**
     * Gibt einen belegten Speicherbereich wieder frei.
     */
    virtual void release(caddr_t memadr) = 0;

    /**
     * ndert die Gre eines Speicherbereichs oder legt ihn neu an and
     * bernimmt den bisherigen Inhalt.
     */
    void* resize(void *ptr, size_t size);    
public:
    MemoryStrategy();
    virtual ~MemoryStrategy();
        
    /**
     * Initialisiert die Verwaltungsstruktur der Speicherstrategie.
     */
    virtual void manage(caddr_t memadr, size_t memsize, size_t alignment = 2) = 0;
    
    /**
     * The malloc() function allocates size bytes of memory. The allocated
     * space is suitably aligned (after possible pointer coercion) for storage
     * of any type of object. If the space is at least pagesize bytes in length
     * (see getpagesize()), the returned memory will be page boundary aligned
     * as well. If malloc() fails, a NULL pointer is returned.
     *
     * <strong>Note</strong> that malloc() does NOT normally initialize the 
     * returned memory to zero bytes.
     *
     * @param size the size of memory to be allocated in bytes
     * @return the address of the allocated memory or NULL if malloc() fails
     */
    void* malloc(size_t size);
    
    /**
     * The calloc() function allocates space for number objects, each size bytes
     * in length. The result is identical to calling malloc() with an argument
     * of <em>"number * size"</em>, with the exception that the allocated memory is
     * explicitly initialized to zero bytes.
     *
     * @param number the number of memory pieces to be allocated
     * @param size the size of memory to be allocated in bytes
     * @return the address of the allocated memory or NULL if calloc() fails
     */
    void* calloc(size_t number, size_t size);
    
    /**
     * The realloc() function changes the size of previously allocated memory 
     * referenced by ptr to size bytes. The contents of the memory are
     * unchanged up to the lesser of the new and old sizes.  If the new size is
     * larger, the value of the newly allocated portion of the memory is 
     * undefined. If the requested memory cannot be allocated, NULL is returned
     * and the memory referenced by ptr is valid and unchanged. If memory can
     * be allocated, the memory referenced by ptr is freed and a pointer to the
     * newly allocated memory is returned. <strong>Note</strong> that this may
     * be different from the value passed as ptr. If ptr is NULL, the realloc()
     * function behaves identically to malloc() for the specified size.
     *
     * @param ptr address of memory segment
     * @param size the new size of memory in bytes
     * @return the address of the reallocated memory or NULL if realloc() fails
     */
    void* realloc(void *ptr, size_t size);
    
    /**
     * The free() function causes the allocated memory referenced by ptr to be
     * made available for future allocations. If ptr is NULL, no action occurs.
     *
     * @param ptr address of memory segment
     */
    void free(void *ptr);
    
    /**
     * Gibt die aktuelle Freispeicherlist auf dem 'ErrorStream' aus. 
     * Diese Methode dient nur zum Debuggen und Testen der Speicherstrategieen.
     */
    virtual void showBlocks() = 0;
    
    /**
     * Gibt die Gre des zur Verfgung stehenden Speichers zurck.
     *
     * @return Liefert die Gre des verwalteten Speichers.
     */
    size_t size();
    
    /**
     * Gibt die Gre des belegten Speichers zurck.
     *
     * @return Liefert die Gre des belegten Speichers
     */
    size_t used();
};

#endif // _MemoryStrategy_interface_
