#ifndef __MemoryList_interface_
#define __MemoryList_interface_

#include <stdio.h>
#include "MemoryStrategy.hh"

#define __single__
#ifdef __single__
#include "SegmentSingle.hh"
typedef SegmentSingle element;
#endif
#ifndef __single__
#include "SegmentDouble.hh"
typedef SegmentDouble element;
#endif

/**
 * Setzt eine Speicherstrategie anhand einer verketteten Liste um.
 * Die Klasse <code>MemoryList</code> wird zur Umsetzung der Strategien:
 * FirstFit, NextFit, BestFit und WorstFit bentigt.
 */
class MemoryList : public MemoryStrategy {
protected:
    /**
     * Wenn <code>_ordered</code> gesetzt ist, wird die Freispeicherliste nach Blockgroesse sortiert.
     */
    bool _ordered;
    
    /**
     * Wurzelknoten der Freispeicherliste
     */
    element* root;
    
    /**
     * Sucht fuer einen Speicherblock den freien Speicherbereich, der genau hinter dem Speicherblock liegt.
     * 
     * @param item Speicherblock fuer den gesucht werden soll
     * @return Liefert den freien Speicherblock, der genau hinter dem zu testenden Block beginnt, oder NULL.
     */
    element* next(element*);
    
    /**
     * Sucht fuer einen Speicherblock den freien Speicherbereich, der genau vor dem Speicherblock liegt.
     * 
     * @param item Speicherblock fuer den gesucht werden soll
     * @return Liefert den freien Speicherblock, der genau vor dem zu testenden Block endet, oder NULL.
     */
    element* prev(element*);
    
    /**
     * Vergleicht zwei Speicherbloecke miteinander. Zum Vergleich wird die Speicheradresse verwendet.
     * Um die Reihenfolge in der Speicherbloecke in die Freispeicherliste eingefuegt werden, 
     * zu aendern, muss diese Methode ueberschrieben werden.
     */
    bool compare(element* a, element* b);
    
    /**
     * Fasst Speicherbloecke, die einen zusammenhaengenden Speicherbereich bilden, zusammen und fuegt den so
     * entstandenen Speicherblock in die Liste der freien Speicherbloecke ein.
     *
     * @param item Ausgangsblock fuer die Zusammenfassung
     */
    void fuse(element* item);
public: 
    MemoryList();
    MemoryList& operator=(const MemoryList&);

    /**
     * Koppelt einen Speicherblock aus der Liste der freien Speicherbereiche aus.
     *
     * @param item Speicherblock, der ausgekoppelt werden soll
     */
    void erase(element* item);
    
    /**
     * Fuegt einen Speicherblock in einer bestimmten Reihenfolge in die Liste der 
     * freien Speicherbereiche ein. Zu diesem Zweck werden die Bloecke mit Hilfe
     * der Methode <code>compare(element*, element*)</code> sortiert.
     *
     * @param item Speicherblock der eingefuegt werden soll
     */
    void insert(element* item);
    
    /**
     * Initialisiert die Liste der freien Speicherbloecke.
     */
    void manage(caddr_t memadr, size_t memsize, size_t alignment = 2);
    
    /**
     * Gibt die aktuelle Freispeicherlist auf dem 'ErrorStream' aus. 
     * Diese Methode dient nur zum Debuggen und Testen der Speicherstrategieen.
     */
    void showBlocks();

    /**
     * Gibt einen reservierten Speicherblock frei. 
     * Die Freigabe erfolgt, indem der Speicherblock wider in die Liste der freien 
     * Speicherbereiche eingefuegt wird. Damit der Speicher nicht fragmentiert, muessen
     * Bloecke, die einen zusammenhaengenden Block bilden zu einem Block zusammengefasst 
     * werden (siehe <code>fuse(element*)</code>).
     *
     * @param adress effektive Speicheradresse des freizugebenden Blocks
     */
    void release(caddr_t memadr);
    
    /**
     * Reserviert einen Speicherblock, mit mindestens der Groesse size, aus der Liste 
     * der freien Speicherbereiche.
     *
     * @param size Groesse des angeforderten Speicherbereiches
     * @return Liefert die Speicheradresse, ab welcher der angeforderte Speicherblock vom Anwender 
     *         benutzt werden kann oder NULL.
     */
    void* get(size_t size);        
};

#endif // __MemoryList_interface
